%% Visual Clustering of Parallel Coordinates
% @author Mazurek/Bartalsky
%% computeForce.m parameters
% * *Description:*
% * Computed the gravitational Force of the parallel coordinates.
% * The computation of the force on a point computed, additionally
% * the direction of the force is determined.
% * *Input:*  
% * _M_ ... parallel coordinates of a dataset, rows are
% dimensions, collumns are observations
% * _subM_ ... subcoordinates of the matrix M
% * _nl_ ... number of neighboring lines considered for the computation
% * _q_alpha_ ... exponent of the angle, strengthens the influence of
% angles on the force
% * _q_d_ ... exponent of the distance, strengthens the influence of
% distance on the force
% * *Ouput:*
% * _fM_ ... direction of the subcoordinates, gravitational force
% * _iM_ ... gravitational force of the subcoordinates 
function [ fM, iM ] = computeForce(M, subM, nl, q_alpha, q_d)


subs=subM(1).subs;
sub=size(subs,2);
fM=zeros(size(M,1),size(M,2)+(size(M,2)-1)*sub);
iM=zeros(size(M,1),size(M,2)+(size(M,2)-1)*sub);

%% computation of the gravitational force between two coordinates
for a=1 : size(M,2)-1
    M2dim=cat(2,M(:,a),subM(a).subs,M(:,a+1));
    [fij2dM, initf2dM]= computeForce2dim(M2dim,nl,q_alpha, q_d);
    fM(:,1+(a-1)*(sub+1):1+(a)*(sub+1))=fij2dM;
    iM(:,1+(a-1)*(sub+1):1+(a)*(sub+1))=initf2dM;
end

end

%% computeAngle parameters
% * *Description:*
% * The angle between two lines is computed by the dot product of the
% * lines. In the further computations the cosine of the angle is used
% * because the possible results lie between 0 and 1. 
% * *Input:*  
% * _li_ ... line at the position that is computed
% * _lk_ ... line of a neighbor
% * *Ouput:*
% * _angle_ ... angle between both lines
function[ angle ]= computeAngle(li,lk)

cos=dot(li,lk);
angle=cos;
%angle=acosd(cos);
if(angle<0.0001)
    angle=1;
end
end

%% computeDistance parameters
% * *Description:*
% * The distance between two points is the absolute difference of the two.
% * Additionally the direction of the force is determined, when the point is 
% * pulled downwards the result is -1, otherwise it is 1.
% * *Input:*  
% * _pij_ ... subcoordinate of the computed position
% * _pkj_ ... subcoordinate of a neighbor
% * *Ouput:*
% * _absl_ ... absolute distance between pij, pkj
% * _direction_ ... direction of the force, depends on where the neighbor
% is positioned
function[ absl, direction ]= computeDistance(pij,pkj)

absl=abs(pij-pkj);
if(absl ==0)
    absl=1;
end

direction=pkj-pij;
if (direction==0)
    direction=1;
end

end

%% computeForce2dim parameters
% * *Description:*
% * The force is computed between two dimensions, for the computation
% * a defined number of neighbors are considered, as well as all lines
% * that intersect with the computed line. For the angular part of the
% force the cosine of the angle is used. The distance part is detemined by
% the absolute difference between two lines and the direction of the force.
% * *Input:*  
% * _M_ ... two dimensions of the parallel coordinates, rows are
% dimensions, collumns are observations, subcoordinates are between the
% dimensions
% * _nl_ ... number of neighboring lines considered for the computation
% * _q_alpha_ ... exponent of the angle, strengthens the influence of
% angles on the force
% * _q_d_ ... exponent of the distance, strengthens the influence of
% distance on the force
% * *Ouput:*
% * _fijmatrix_ ... direction of the subcoordinates, gravitational force
% * _initfmatrix_ ... gravitational force of the subcoordinates 
function[ fijmatrix, initfmatrix ]= computeForce2dim(M, nl, q_alpha, q_d)

%sort rows descending to get indizes of all neighbors
numbers=linspace(1,size(M,1),size(M,1))';
sortedM1=cat(2,numbers,M);
sortedM1= sortrows(sortedM1,-2);
col1=sortedM1(:,1);
sortedM2=cat(2,numbers,M);
sortedM2= sortrows(sortedM2,-size(sortedM2,2));
col2=sortedM2(:,1);

%define all lines as linesegments for intersectioncontrol
XY1=[zeros(size(M,1),1), M(:,1), ones(size(M,1),1), M(:,end)];
out=lineSegmentIntersect(XY1,XY1);
imatrix=out.intAdjacencyMatrix;

fijmatrix=zeros(size(M,1),size(M,2));
initfmatrix=zeros(size(M,1),size(M,2));
% compute Fij
for i=1:size(M,1) 
    
    index1=find(col1==i);
    index2=find(col2==i);
    
    %% computation of neighboring edges    
    neighbors=zeros(nl*4,1);
    count=1;
    for b=1:nl
        if index1-b>0 && count<=nl*4
            if all(sortedM1(index1-b,1)~=neighbors)
                %all neighoring lines above computed line
                neighbors(count)=sortedM1(index1-b,1);
                count=count+1;
            end
        end
        if index1+b<=size(M,1) && count<=nl*4
            if all(sortedM1(index1+b,1)~=neighbors)
                %all neighoring lines below computed line
                neighbors(count)=sortedM1(index1+b,1);
                count=count+1;
            end
        end
        if index2-b>0 && count<=nl*4
            if all(sortedM2(index2-b,1)~=neighbors)
                %all neighoring lines above computed line
                neighbors(count)=sortedM2(index2-b,1);
                count=count+1;
            end
        end
        if index2+b<=size(M,1) && count<=nl*4
            if all(sortedM2(index2+b,1)~=neighbors)
                %all neighoring lines below computed line
                neighbors(count)=sortedM2(index2+b,1);
                count=count+1;
            end
        end 
    end
    neighbors(neighbors==0)=[];
    
    %% computation of intersecting lines
    intersections=zeros(size(M,1),1);
    count=1;
    for c=1:size(M,1)
        if any(c==neighbors) || c==i
        elseif imatrix(i,c)==1
        intersections(count)=c;
        count=count+1;
        end
    end
    intersections(intersections==0)=[];

    nedges=cat(1,neighbors,intersections);
    
    %% computation: all Fij of a certain line 
    
    li=[1,M(i,end)]-[0,M(i,1)]; 
    for j=2:size(M,2)-1 
        sumFij=0;
        sumFij2=0;
        for k=1:size(nedges,1)
        
            %% computation of angle
            lk=[1,M(nedges(k),end)]-[0,M(nedges(k),1)];
            nli=li/norm(li);
            nlk=lk/norm(lk);
            angle=computeAngle(nli,nlk);
            
            %% computation of distance
            pij=M(i,j);
            pkj=M(nedges(k),j);
            [absl, direction]=computeDistance(pij,pkj);
            
            if(direction>=1)
                flilk=(1/(angle^q_alpha))*(1/(direction^q_d));
            else
                flilk=(1/(angle^q_alpha))*(1/(direction^q_d));
            end
            
            %% summ the components up for Fij
            sumFij=sumFij+flilk;
            flilk2=(1/(angle^q_alpha))*(1/(absl^q_d));
            %% summ the components up for Fij
            sumFij2=sumFij2+flilk2;
        end
        %% determine direction
        if sumFij==0
            fijmatrix(i,j)=0; 
        elseif sumFij<1
            fijmatrix(i,j)=1;
        else
            fijmatrix(i,j)=-1;
        end
        
        initfmatrix(i,j)=sumFij2;
    end    
end

end
