%% Generating the Equations
% @author Mazurek/Bartalsky
% This method is generating the optimization-problem equation between two
% coordinates.
%% generateEQ.m parameters
%
% * *Input:*  
% * _M_ ... two neighbouring parallel coordinates of a dataset
% * _m_ ... number of subCoordinates
% * _alpha_c_ ... The share of the two energy terms.
% * _forceM_ ... The computed force.
% * _initFM_ ... The initial force position.
% * *Ouput:*
% * _output_ ... the computed subCoords after the computation

function [ output ] = generateEQ(M, subM, forceM, initFM, alpha_c)

% Step 1 - Get the descend order of the subCoords in another matrix
% These needs to be done to set the constraints correctly, otherwise the
% order wouldn't be guaranteed. We need the indices of the correct order.

[subs, indices] = sort(subM,1,'descend');

% The indices matrix shows us the constraints. In the descending order, the
% first must be higherOrequal than the second, ... . This order has to be
% now interpreted in lp_solve as constraints for each subcoords column,
% independent from the other subcoords columns.

m = size(subM,2);
n = size(subM,1);

rh = 0;
f = zeros(m, n);
A = 0;
% This loop iterates through the columns and adds it to the function f.
% Furthermore it prepares the constraint matrix A, which checks the
% boundings of the EQ.
for j=1:m
    for i=1:n
        % This is the E_Gravitation term:
        % f(j,i) and not f(i,j) because the variable names start from
        % x1,x2 column by column (and not row by row)
        f(j,i) = forceM(i,j)*initFM(i,j);
        rh = rh + initFM(i,j)*subM(i,j);
        % This part generates the constraint Matrix, which is E_ij;
        if(i < n)
            temp = zeros(1, n*m);
            
            temp((j-1)*n + indices(i,j)) = 1;
            temp((j-1)*n + indices(i+1,j)) = -1;

            if(A == 0)
              A = temp; 
            else 
              A = [A;temp];
            end
        end
    end
end

% the constraint for the object function
temp=(-1)*ones(1, n*m);
A = [A;temp];

%Make one long row. (format)
temp = 0;
for i=1:m
    if(i == 1)
       temp = f(i,:); 
    else
        temp = [temp f(i,:)];
    end  
end

B=reshape(subM,numel(subM),1);
C=reshape(forceM,numel(forceM),1);

% Set the bounds of each variable L and U
Lt=zeros(size(C));
Ut=ones(size(C));
L= B-(abs(C).*(1-alpha_c));
LB=L;
LB(Lt>L)=0;
U= B+(abs(C).*(1-alpha_c));
UB=U;
UB(Ut<U)=1;

f = temp;
b = zeros(size(A,1),1);
e = ones(size(A,1),1);
b(end)=-rh;


% This function creates the optimization problem for lp_solve.
lp = lp_maker(f, A, b, e, LB, UB, [], 0, 1, rh);
% f ... is the function
% A ... is the constraint matrix, which guarantees that the non-intersecting
% sample points do not cross
% b ... is the right side of the constraints (in our case it is always 0)
% e ... is the definition, if it is >, = or < 
% 3x [] bounderies (lowerbound, upperbound, integer)
% two last paramters: scale_0 and 1 because of finding the minimum
% rh is the right side of the objective function


% lp_solve commands for computing the optimization problem and getting the
% variables.
solvestat = mxlpsolve('solve', lp);
variables = mxlpsolve('get_variables', lp);
constraints = mxlpsolve('get_constraints', lp);
objective = mxlpsolve('get_objective',lp);


% Put the results into the previous subCoords shape.
output = reshape(variables,[n,m]);

% deallocate
mxlpsolve('delete_lp', lp);

end

